@@ -659,9 +659,7 @@ disk_name_data_func (GtkCellLayout   *cell_layout,
         gchar *name;
         gchar *vpd_name;
         gchar *markup;
-        GtkStyle *style;
-        GdkColor desc_gdk_color = {0};
-        gchar *desc_color;
+        gchar color[16];
         GtkStateType state;
         gboolean sensitive;
         GduPresentable *p;
@@ -678,34 +676,21 @@ disk_name_data_func (GtkCellLayout   *cell_layout,
         if (p != NULL && GDU_IS_DRIVE (p))
                 sensitive = is_drive_selectable (widget, GDU_DRIVE (p), NULL);
 
-        /* This color business shouldn't be this hard... */
-        style = gtk_widget_get_style (GTK_WIDGET (widget->priv->tree_view));
         if (gtk_tree_selection_iter_is_selected (tree_selection, iter)) {
-                if (GTK_WIDGET_HAS_FOCUS (GTK_WIDGET (widget->priv->tree_view)))
+                if (gtk_widget_has_focus (GTK_WIDGET (widget->priv->tree_view)))
                         state = GTK_STATE_SELECTED;
                 else
                         state = GTK_STATE_ACTIVE;
         } else {
                 state = GTK_STATE_NORMAL;
         }
-#define BLEND_FACTOR 0.7
-        desc_gdk_color.red   = style->text[state].red   * BLEND_FACTOR +
-                               style->base[state].red   * (1.0 - BLEND_FACTOR);
-        desc_gdk_color.green = style->text[state].green * BLEND_FACTOR +
-                               style->base[state].green * (1.0 - BLEND_FACTOR);
-        desc_gdk_color.blue  = style->text[state].blue  * BLEND_FACTOR +
-                               style->base[state].blue  * (1.0 - BLEND_FACTOR);
-#undef BLEND_FACTOR
-        desc_color = g_strdup_printf ("#%02x%02x%02x",
-                                      (desc_gdk_color.red >> 8),
-                                      (desc_gdk_color.green >> 8),
-                                      (desc_gdk_color.blue >> 8));
+        gdu_util_get_mix_color (GTK_WIDGET (widget->priv->tree_view), state, color, sizeof (color));
 
         if (sensitive) {
                 markup = g_strdup_printf ("<small><b>%s</b>\n"
                                           "<span fgcolor=\"%s\">%s</span></small>",
                                           name,
-                                          desc_color,
+                                          color,
                                           vpd_name);
         } else {
                 markup = g_strdup_printf ("<small><b>%s</b>\n"
@@ -724,7 +709,6 @@ disk_name_data_func (GtkCellLayout   *cell_layout,
         g_free (name);
         g_free (vpd_name);
         g_free (markup);
-        g_free (desc_color);
 
         if (p != NULL)
                 g_object_unref (p);
