@@ -25,7 +25,6 @@
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
-#include <X11/XKBlib.h>
 
 #include <gdu-gtk/gdu-gtk.h>
 
@@ -128,8 +127,8 @@ static GridElement *find_element_for_position (GduVolumeGrid *grid,
                                                guint x,
                                                guint y);
 
-static gboolean gdu_volume_grid_expose_event (GtkWidget           *widget,
-                                              GdkEventExpose      *event);
+static gboolean gdu_volume_grid_draw (GtkWidget *widget,
+                                      cairo_t   *cr);
 
 static void on_presentable_added        (GduPool        *pool,
                                          GduPresentable *p,
@@ -224,10 +223,6 @@ gdu_volume_grid_constructed (GObject *object)
 {
         GduVolumeGrid *grid = GDU_VOLUME_GRID (object);
 
-        gtk_widget_set_size_request (GTK_WIDGET (grid),
-                                     -1,
-                                     100);
-
         g_signal_connect (grid->priv->pool,
                           "presentable-added",
                           G_CALLBACK (on_presentable_added),
@@ -258,26 +253,6 @@ gdu_volume_grid_constructed (GObject *object)
                 G_OBJECT_CLASS (gdu_volume_grid_parent_class)->constructed (object);
 }
 
-static gboolean
-is_ctrl_pressed (void)
-{
-        gboolean ret;
-        XkbStateRec state;
-        Bool status;
-
-        ret = FALSE;
-
-        gdk_error_trap_push ();
-        status = XkbGetState (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), XkbUseCoreKbd, &state);
-        gdk_error_trap_pop ();
-
-        if (status == Success) {
-                ret = ((state.mods & ControlMask) != 0);
-        }
-
-        return ret;
-}
-
 static gboolean
 gdu_volume_grid_key_press_event (GtkWidget      *widget,
                                  GdkEventKey    *event)
@@ -292,10 +267,10 @@ gdu_volume_grid_key_press_event (GtkWidget      *widget,
                 goto out;
 
         switch (event->keyval) {
-        case GDK_Left:
-        case GDK_Right:
-        case GDK_Up:
-        case GDK_Down:
+        case GDK_KEY_Left:
+        case GDK_KEY_Right:
+        case GDK_KEY_Up:
+        case GDK_KEY_Down:
                 target = NULL;
 
                 if (grid->priv->focused == NULL) {
@@ -305,25 +280,25 @@ gdu_volume_grid_key_press_event (GtkWidget      *widget,
 
                         element = grid->priv->focused;
                         if (element != NULL) {
-                                if (event->keyval == GDK_Left) {
+                                if (event->keyval == GDK_KEY_Left) {
                                         if (element->prev != NULL) {
                                                 target = element->prev;
                                         } else {
                                                 if (element->parent && element->parent->prev != NULL)
                                                         target = element->parent->prev;
                                         }
-                                } else if (event->keyval == GDK_Right) {
+                                } else if (event->keyval == GDK_KEY_Right) {
                                         if (element->next != NULL) {
                                                 target = element->next;
                                         } else {
                                                 if (element->parent && element->parent->next != NULL)
                                                         target = element->parent->next;
                                         }
-                                } else if (event->keyval == GDK_Up) {
+                                } else if (event->keyval == GDK_KEY_Up) {
                                         if (element->parent != NULL) {
                                                 target = element->parent;
                                         }
-                                } else if (event->keyval == GDK_Down) {
+                                } else if (event->keyval == GDK_KEY_Down) {
                                         if (element->embedded_elements != NULL) {
                                                 target = (GridElement *) element->embedded_elements->data;
                                         }
@@ -332,7 +307,7 @@ gdu_volume_grid_key_press_event (GtkWidget      *widget,
                 }
 
                 if (target != NULL) {
-                        if (is_ctrl_pressed ()) {
+                        if ((event->state & GDK_CONTROL_MASK) != 0) {
                                 grid->priv->focused = target;
                         } else {
                                 grid->priv->selected = target;
@@ -346,8 +321,8 @@ gdu_volume_grid_key_press_event (GtkWidget      *widget,
                 handled = TRUE;
                 break;
 
-        case GDK_Return:
-        case GDK_space:
+        case GDK_KEY_Return:
+        case GDK_KEY_space:
                 if (grid->priv->focused != grid->priv->selected &&
                     grid->priv->focused != NULL) {
                         grid->priv->selected = grid->priv->focused;
@@ -411,6 +386,7 @@ gdu_volume_grid_realize (GtkWidget *widget)
         GdkWindowAttr attributes;
         gint attributes_mask;
         GtkAllocation allocation;
+        GtkStyleContext *context;
 
         gtk_widget_set_realized (widget, TRUE);
         gtk_widget_get_allocation (widget, &allocation);
@@ -429,9 +405,8 @@ gdu_volume_grid_realize (GtkWidget *widget)
                 GDK_ENTER_NOTIFY_MASK |
                 GDK_LEAVE_NOTIFY_MASK;
         attributes.visual = gtk_widget_get_visual (widget);
-        attributes.colormap = gtk_widget_get_colormap (widget);
 
-        attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+        attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
 
         window = gtk_widget_get_parent_window (widget);
         gtk_widget_set_window (widget, window);
@@ -443,10 +418,8 @@ gdu_volume_grid_realize (GtkWidget *widget)
         gtk_widget_set_window (widget, window);
         gdk_window_set_user_data (window, grid);
 
-        gtk_widget_style_attach (widget);
-        gtk_style_set_background (gtk_widget_get_style (widget),
-                                  window,
-                                  GTK_STATE_NORMAL);
+        context = gtk_widget_get_style_context (widget);
+        gtk_style_context_set_background (context, window);
 }
 
 static guint
@@ -467,16 +440,27 @@ get_num_elements_for_slice (GList *elements)
                 return 1;
 }
 
+static void
+gdu_volume_grid_get_preferred_width (GtkWidget *widget,
+                                     gint      *minimal_width,
+                                     gint      *natural_width)
+{
+  GduVolumeGrid *grid = GDU_VOLUME_GRID (widget);
+  guint num_elements;
+  gint width;
+
+  num_elements = get_num_elements_for_slice (grid->priv->elements);
+  width = num_elements * ELEMENT_MINIMUM_WIDTH;
+
+  *minimal_width = *natural_width = width;
+}
 
 static void
-gdu_volume_grid_size_request (GtkWidget      *widget,
-                              GtkRequisition *requisition)
+gdu_volume_grid_get_preferred_height (GtkWidget *widget,
+                                      gint      *minimal_height,
+                                      gint      *natural_height)
 {
-        GduVolumeGrid *grid = GDU_VOLUME_GRID (widget);
-        guint num_elements;
-
-        num_elements = get_num_elements_for_slice (grid->priv->elements);
-        requisition->width = num_elements * ELEMENT_MINIMUM_WIDTH;
+  *minimal_height = *natural_height = 100;
 }
 
 static void
@@ -492,11 +476,12 @@ gdu_volume_grid_class_init (GduVolumeGridClass *klass)
         object_class->constructed  = gdu_volume_grid_constructed;
         object_class->finalize     = gdu_volume_grid_finalize;
 
-        widget_class->realize            = gdu_volume_grid_realize;
-        widget_class->key_press_event    = gdu_volume_grid_key_press_event;
-        widget_class->button_press_event = gdu_volume_grid_button_press_event;
-        widget_class->size_request       = gdu_volume_grid_size_request;
-        widget_class->expose_event       = gdu_volume_grid_expose_event;
+        widget_class->realize              = gdu_volume_grid_realize;
+        widget_class->key_press_event      = gdu_volume_grid_key_press_event;
+        widget_class->button_press_event   = gdu_volume_grid_button_press_event;
+        widget_class->get_preferred_width  = gdu_volume_grid_get_preferred_width;
+        widget_class->get_preferred_height = gdu_volume_grid_get_preferred_height;
+        widget_class->draw                 = gdu_volume_grid_draw;
 
         g_object_class_install_property (object_class,
                                          PROP_DRIVE,
@@ -1038,54 +1023,49 @@ render_element (GduVolumeGrid *grid,
         gdouble text_selected_not_focused_red;
         gdouble text_selected_not_focused_green;
         gdouble text_selected_not_focused_blue;
-        GtkStyle *style;
         PangoLayout *layout;
         PangoFontDescription *desc;
         gint width, height;
 
         need_animation_timeout = FALSE;
 
-        style = gtk_widget_get_style (GTK_WIDGET (grid));
-
-        fill_red   = style->base[GTK_STATE_NORMAL].red   / 65535.0;
-        fill_green = style->base[GTK_STATE_NORMAL].green / 65535.0;
-        fill_blue  = style->base[GTK_STATE_NORMAL].blue  / 65535.0;
-        fill_selected_red   = style->base[GTK_STATE_SELECTED].red   / 65535.0;
-        fill_selected_green = style->base[GTK_STATE_SELECTED].green / 65535.0;
-        fill_selected_blue  = style->base[GTK_STATE_SELECTED].blue  / 65535.0;
-        fill_selected_not_focused_red   = style->base[GTK_STATE_ACTIVE].red   / 65535.0;
-        fill_selected_not_focused_green = style->base[GTK_STATE_ACTIVE].green / 65535.0;
-        fill_selected_not_focused_blue  = style->base[GTK_STATE_ACTIVE].blue  / 65535.0;
-
-        stroke_red   = fill_red   * 0.75;
-        stroke_green = fill_green * 0.75;
-        stroke_blue  = fill_blue  * 0.75;
-        stroke_selected_red   = fill_selected_red   * 0.75;
-        stroke_selected_green = fill_selected_green * 0.75;
-        stroke_selected_blue  = fill_selected_blue  * 0.75;
-        stroke_selected_not_focused_red   = fill_selected_not_focused_red   * 0.75;
-        stroke_selected_not_focused_green = fill_selected_not_focused_green * 0.75;
-        stroke_selected_not_focused_blue  = fill_selected_not_focused_blue  * 0.75;
-
-        focus_rect_red   = style->text_aa[GTK_STATE_NORMAL].red   / 65535.0;
-        focus_rect_green = style->text_aa[GTK_STATE_NORMAL].green / 65535.0;
-        focus_rect_blue  = style->text_aa[GTK_STATE_NORMAL].blue  / 65535.0;
-        focus_rect_selected_red   = style->text_aa[GTK_STATE_SELECTED].red   / 65535.0;
-        focus_rect_selected_green = style->text_aa[GTK_STATE_SELECTED].green / 65535.0;
-        focus_rect_selected_blue  = style->text_aa[GTK_STATE_SELECTED].blue  / 65535.0;
-        focus_rect_selected_not_focused_red   = style->text_aa[GTK_STATE_ACTIVE].red   / 65535.0;
-        focus_rect_selected_not_focused_green = style->text_aa[GTK_STATE_ACTIVE].green / 65535.0;
-        focus_rect_selected_not_focused_blue  = style->text_aa[GTK_STATE_ACTIVE].blue  / 65535.0;
-
-        text_red   = style->fg[GTK_STATE_NORMAL].red   / 65535.0;
-        text_green = style->fg[GTK_STATE_NORMAL].green / 65535.0;
-        text_blue  = style->fg[GTK_STATE_NORMAL].blue  / 65535.0;
-        text_selected_red   = style->fg[GTK_STATE_SELECTED].red   / 65535.0;
-        text_selected_green = style->fg[GTK_STATE_SELECTED].green / 65535.0;
-        text_selected_blue  = style->fg[GTK_STATE_SELECTED].blue  / 65535.0;
-        text_selected_not_focused_red   = style->fg[GTK_STATE_ACTIVE].red   / 65535.0;
-        text_selected_not_focused_green = style->fg[GTK_STATE_ACTIVE].green / 65535.0;
-        text_selected_not_focused_blue  = style->fg[GTK_STATE_ACTIVE].blue  / 65535.0;
+        /* TODO: use GtkStyleContext and/or CSS etc. instead of hard-coding colors */
+        fill_red     = 1;
+        fill_green   = 1;
+        fill_blue    = 1;
+        fill_selected_red     = 0.40;
+        fill_selected_green   = 0.60;
+        fill_selected_blue    = 0.80;
+        fill_selected_not_focused_red     = 0.60;
+        fill_selected_not_focused_green   = 0.60;
+        fill_selected_not_focused_blue    = 0.60;
+        focus_rect_red     = 0.75;
+        focus_rect_green   = 0.75;
+        focus_rect_blue    = 0.75;
+        focus_rect_selected_red     = 0.70;
+        focus_rect_selected_green   = 0.70;
+        focus_rect_selected_blue    = 0.80;
+        focus_rect_selected_not_focused_red     = 0.70;
+        focus_rect_selected_not_focused_green   = 0.70;
+        focus_rect_selected_not_focused_blue    = 0.70;
+        stroke_red   = 0.75;
+        stroke_green = 0.75;
+        stroke_blue  = 0.75;
+        stroke_selected_red   = 0.3;
+        stroke_selected_green = 0.45;
+        stroke_selected_blue  = 0.6;
+        stroke_selected_not_focused_red   = 0.45;
+        stroke_selected_not_focused_green = 0.45;
+        stroke_selected_not_focused_blue  = 0.45;
+        text_red     = 0;
+        text_green   = 0;
+        text_blue    = 0;
+        text_selected_red     = 1;
+        text_selected_green   = 1;
+        text_selected_blue    = 1;
+        text_selected_not_focused_red     = 1;
+        text_selected_not_focused_green   = 1;
+        text_selected_not_focused_blue    = 1;
 
 #if 0
         g_debug ("rendering element: x=%d w=%d",
@@ -1476,12 +1456,11 @@ render_slice (GduVolumeGrid *grid,
 }
 
 static gboolean
-gdu_volume_grid_expose_event (GtkWidget           *widget,
-                              GdkEventExpose      *event)
+gdu_volume_grid_draw (GtkWidget *widget,
+                      cairo_t   *cr)
 {
         GduVolumeGrid *grid = GDU_VOLUME_GRID (widget);
         GtkAllocation allocation;
-        cairo_t *cr;
         gdouble width;
         gdouble height;
         gboolean need_animation_timeout;
@@ -1494,16 +1473,8 @@ gdu_volume_grid_expose_event (GtkWidget           *widget,
                         width - 1,
                         height -1);
 
-        cr = gdk_cairo_create (gtk_widget_get_window (widget));
-        cairo_rectangle (cr,
-                         event->area.x, event->area.y,
-                         event->area.width, event->area.height);
-        cairo_clip (cr);
-
         need_animation_timeout = render_slice (grid, cr, grid->priv->elements);
 
-        cairo_destroy (cr);
-
         if (need_animation_timeout) {
                 if (grid->priv->animation_timeout_id == 0) {
                         grid->priv->animation_timeout_id = g_timeout_add (80,
